# Copyright (C) 2018 Avatao.com Innovative Learning Kft.
# All Rights Reserved. See LICENSE file for details.

import zmq
from zmq.eventloop.zmqstream import ZMQStream

from tfw.config import TFWENV
from tfw.config.logs import logging

from .serialization import serialize_tfw_msg, with_deserialize_tfw_msg

LOG = logging.getLogger(__name__)


class EventHandlerDownlinkConnector():
    def __init__(self):
        self._zmq_pull_socket = zmq.Context.instance().socket(zmq.PULL)
        self._zmq_pull_socket.setsockopt(zmq.RCVHWM, 0)
        self._zmq_pull_stream = ZMQStream(self._zmq_pull_socket)
        address = f'tcp://*:{TFWENV.PULL_PORT}'
        self._zmq_pull_socket.bind(address)
        LOG.debug('Pull socket bound to %s', address)

    def register_callback(self, callback):
        callback = with_deserialize_tfw_msg(callback)
        self._zmq_pull_stream.on_recv(callback)

    def close(self):
        self._zmq_pull_stream.close()


class EventHandlerUplinkConnector():
    def __init__(self):
        self._zmq_pub_socket = zmq.Context.instance().socket(zmq.PUB)
        self._zmq_pub_socket.setsockopt(zmq.SNDHWM, 0)
        address = f'tcp://*:{TFWENV.PUB_PORT}'
        self._zmq_pub_socket.bind(address)
        LOG.debug('Pub socket bound to %s', address)

    def send_message(self, message: dict):
        self._zmq_pub_socket.send_multipart(serialize_tfw_msg(message))

    def close(self):
        self._zmq_pub_socket.close()


class EventHandlerConnector(EventHandlerDownlinkConnector, EventHandlerUplinkConnector):
    def __init__(self):
        EventHandlerDownlinkConnector.__init__(self)
        EventHandlerUplinkConnector.__init__(self)

    def close(self):
        EventHandlerDownlinkConnector.close(self)
        EventHandlerUplinkConnector.close(self)
